;USB Mass Storage Device Driver
; (C) 2005 by Michael Vincent. All rights reserved. 
;Portions in usbflash.asm, util.asm, usb8x.inc, equates.inc (C) Dan Englender.

 include "settings.inc"            ; Specific settings for this application
 NOLIST
 include "ti83plus.inc"
 LIST
 include "equates.inc"             ; Equates and macros to be used
 include "header.asm"          
 GLOBALS ON
 include "usb8x.inc"            ;usb8x equates


 SEGMENT MAIN

 EXTERN DispHexA,IGetKey,IPutS,IPutC,DispHexHL

 Var USBDriverCall,32              ;RAM calls for U_CALL and Callback routine
 Var USBDriverBuf,128              ;Static buffer for USB driver's usage
 Var DescBuf,128                   ;Temporary buffer to hold descriptors
 Var KBDBuf,6
 Var BulkBuf, 31							;BOT - used to build CBWs
 Var PacketData,64                  ;Received (or sent) data...bulk-in
 Var BOTState,1							;Byte indicating the current state
 Var LBA,4								;32-bit, LBA input for some UFI commands, LSB first
 Var PartStart,4						;Starting LBA block of partition to use
 Var PartSize,4						;Number of LBA blocks in partition
 Var BPS,2								;Number of bytes per sector of the flash device
 Var DataInAddr,2						;Location to put received data packets
 Var DataInExpected,2				;Number of bytes of data expected in bulk-in endpoint
 Var SectorBuffer,2					;Location to store sectors (512 bytes)
 Var SPC,1								;Number of sectors per FAT cluster (1-128, powers of 2)
 Var FATSectors,2						;Count of total sectors in the FAT partition
 Var RootDirSectors,2				;from FAT32 specification
 Var FirstDataSector,4				;from FAT32 specification
 Var FirstRootDirSecNum,2			;from FAT32 specification
 Var FATStart,2						;sector that marks the start of the first FAT
_HomeUp EQU 4558h
Init:
	jp Explore
	call initializeMSD
   B_CALL ClrLCDFull
	;Test
	call sendReset
	ld a,errResetErr
	jp c,displayErr
	call UFI_getName
	;Display vendor/product ID
	ld hl,OP1
	B_CALL PutS
	ld hl,OP2
	B_CALL PutS
	call UFI_getCapacity
	ld hl,Megabyte
	B_CALL Mov9ToOP2
	B_CALL FPDiv
	ld a,5
	B_CALL FormReal
	ld hl,OP3
	B_CALL PutS
	ld de,savesscreen
	call UFI_Initialize
	call FAT_Initialize
	call IGetKey
	cp kEnter
	jr nz,$F
	ld hl,(0C000h)
	ld de,3Fh
	add hl,de
	;
	ld hl,01F8h
	ld de,3Fh
	add hl,de
	;
	ld (LBA),hl
	ld hl,0
	ld (LBA+2),hl
	ld de,savesscreen
	call UFI_Read
	

	ld hl,Root_Directory
	call FAT_lookupPath
	ld de,26
	add hl,de
	ld a,(hl)
	inc hl
	ld h,(hl)
	ld l,a
	call FAT_getSectorOfCluster
	call PartitionAdjust
	ld de,savesscreen
	call UFI_Read
	jp $F
	ld hl,movie
	ld de,0E000h
	call DOS_openFile
	ld a,43h
	out (30h),a
	ld a,1
	out (31h),a
	ld a,2
	out (32h),a
	ld e,a
	ld bc,3749
	ld a,1
	out (20h),a			;15 MHz
movieloop:
	push bc
	push de
	ld hl,0E000h
	ld bc,768
	ld de,plotsscreen
	call DOS_fileRead
	call fastCopy
	B_CALL GetCSC
	cp skClear
	jp z,AppExit
	pop de
movieloop2:
	in a,(32h)
	cp e
	jr z,movieloop2
	ld e,a
	pop bc
	dec bc
	ld a,b
	or c
	jr nz,movieloop
	;ld hl,test
	;ld de,0E000h
	;call DOS_openFile
	;ld hl,0E000h
	;ld de,0C000h
	;ld bc,768
	;call DOS_fileRead
	;ld hl,0E000h
	;ld de,0D000h
	;ld bc,768
	;call DOS_fileRead
	
	;ld de,test
	;ld hl,0
	;call FAT_findFileInCluster
	;ld (0D000h),hl
$$:
   jp AppExit
test:
	DB "/readme.txt",0
movie:
	DB "/matrix.mov",0
Megabyte:
	DB 0,86h,10h,48h,57h,60h,0,0,0
AppExit:
	U_CALL HostKill
	U_CALL DriverKill
	B_CALL JForceCmdNoChar
fastCopy:
	di
	ld hl,plotsscreen
	ld b,64
	ld a,07h
	out (10h),a
	ld a,80h-1
layerloop1:
	push bc
	ld b,0Ch
	inc a
	ld (OP1),a
	call lcddelay
	out (10h),a
	call lcddelay
	ld a,20h
	out (10h),a
layerloop2:
	ld a,(hl)
	call lcddelay
	out (11h),a
	inc hl
	djnz layerloop2
	pop bc
	ld a,(OP1)
	djnz layerloop1
	ld a,5
	call lcddelay
	out (10h),a
	ei
	ret
lcddelay:
	push af
lcddelayloop:
	in a,(2)
	and 2
	jr z,lcddelayloop
	pop af
	ret
USBError:
       B_CALL ClrLCDFull
       B_CALL HomeUp
       ld     hl,ErrorTXT
       call   IPutS
       ld     hl,USBDriverBuf+offErrorCode
       ld     a,(hl)
       call   DispHexA
       inc    hl
       ld     a,(hl)
       call   DispHexA
ErrorTXT:
       db     "ERROR ",0
 INCLUDE "bulk.asm"
 INCLUDE "ufi.asm"
 INCLUDE "err.asm"
 INCLUDE "fat.asm"
 INCLUDE "explore.asm"
